@@ -1,6 +1,6 @@
 #!/bin/sh
 
-# Copyright (C) 2017-2020 Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2019-2020 Internet Systems Consortium, Inc. ("ISC")
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -22,13 +22,13 @@ prefix="/usr/local"
 if [ -e ${prefix}/share/kea/scripts/admin-utils.sh ]; then
     . "${prefix}/share/kea/scripts/admin-utils.sh"
 else
-    . "/usr/obj/ports/kea-2.0.0-postgresql/kea-2.0.0/src/bin/admin/admin-utils.sh"
+    . "/usr/obj/ports/kea-2.0.2-postgresql/kea-2.0.2/src/bin/admin/admin-utils.sh"
 fi
 
 VERSION=$(pgsql_version "$@")
 
-if [ "$VERSION" != "3.0" ]; then
-    printf 'This script upgrades 3.0 to 3.1. '
+if [ "$VERSION" != "5.0" ]; then
+    printf 'This script upgrades 5.0 to 5.1. '
     printf 'Reported version is %s. Skipping upgrade.\n' "${VERSION}"
     exit 0
 fi
@@ -37,27 +37,14 @@ psql "$@" >/dev/null <<EOF
 
 START TRANSACTION;
 
--- Upgrade to schema 3.1 begins here:
+-- Put the auth key in hexadecimal (double size but far more user friendly).
+ALTER TABLE hosts ALTER COLUMN auth_key TYPE VARCHAR(32);
 
--- The 'client-id' host identifier type was missing in the
--- 2.0 -> 3.0 upgrade script. However, it was present in the
--- dhcpdb_create.pgsql file. This means that this entry may
--- or may not be present. By the conditional insert below we
--- will only insert it if it doesn't exist.
-INSERT INTO host_identifier_type (type, name)
-  SELECT 3, 'client-id'
-    WHERE NOT EXISTS (
-        SELECT type FROM host_identifier_type WHERE type = 3
-    );
-
--- We also add a new identifier type: flex-id.
-INSERT INTO host_identifier_type VALUES (4, 'flex-id');
-
--- Set 3.1 schema version.
+-- Set 5.1 schema version.
 UPDATE schema_version
-    SET version = '3', minor = '1';
+    SET version = '5', minor = '1';
 
--- Schema 3.1 specification ends here.
+-- Schema 5.1a specification ends here.
 
 -- Commit the script transaction
 COMMIT;
