@@ -28,7 +28,7 @@ prefix="/usr/local"
 if [ -e ${prefix}/share/kea/scripts/admin-utils.sh ]; then
     . "${prefix}/share/kea/scripts/admin-utils.sh"
 else
-    . "/usr/obj/ports/kea-2.0.0-postgresql/kea-2.0.0/src/bin/admin/admin-utils.sh"
+    . "/pobj/kea-2.0.2-postgresql/kea-2.0.2/src/bin/admin/admin-utils.sh"
 fi
 
 # First argument is must be the expected schema version <major>.<minor>
